@@ -61,6 +61,7 @@
 #include <OpenIPMI/ipmi_mc.h>
 #include <OpenIPMI/ipmi_lan.h>
 #include <OpenIPMI/extcmd.h>
+#include "ipmi_sim.h"
 
 static void ipmi_mc_start_cmd(lmc_data_t *mc);
 
@@ -71,6 +72,19 @@ get_lanserv_version(void)
     return PVERSION;
 }
 
+struct {
+    cmd_handler_f handler;
+    void *cb_data;
+} group_extension_handlers[256];
+
+void
+ipmi_emu_register_group_extension_handler(uint8_t group_extension,
+					  cmd_handler_f handler, void *cb_data)
+{
+    group_extension_handlers[group_extension].handler = handler;
+    group_extension_handlers[group_extension].cb_data = cb_data;
+}
+
 static void
 handle_group_extension_netfn(lmc_data_t    *mc,
 			     msg_t         *msg,
@@ -78,21 +92,17 @@ handle_group_extension_netfn(lmc_data_t    *mc,
 			     unsigned int  *rdata_len,
 			     void          *cb_data)
 {
+    uint8_t ge;
+
     if (check_msg_length(msg, 1, rdata, rdata_len))
 	return;
 
-    switch (msg->data[0]) {
-    case IPMI_PICMG_GRP_EXT:
-	if (mc->emu->atca_mode)
-	    handle_picmg_msg(mc, msg, rdata, rdata_len);
-	else
-	    handle_invalid_cmd(mc, rdata, rdata_len);
-	break;
-
-    default:
+    ge = msg->data[0];
+    if (group_extension_handlers[ge].handler)
+	group_extension_handlers[ge].handler(mc, msg, rdata, rdata_len,
+				     group_extension_handlers[ge].cb_data);
+    else
 	handle_invalid_cmd(mc, rdata, rdata_len);
-	break;
-    }
 }
 
 static struct iana_handler_elem {
@@ -155,14 +165,14 @@ handle_iana_netfn(lmc_data_t    *mc,
     }
 
     /* Remove the IANA */
-    memcpy(msg->data, msg->data + 3, msg->len - 3);
+    memmove(msg->data, msg->data + 3, msg->len - 3);
     msg->len -= 3;
 
     p->handler(mc, msg, rdata, rdata_len, p->cb_data);
 
  out:
     /* Insert the IANA back in. */
-    memcpy(rdata + 4, rdata + 1, *rdata_len);
+    memmove(rdata + 4, rdata + 1, *rdata_len);
     rdata[1] = msg->iana & 0xff;
     rdata[2] = (msg->iana >> 8) & 0xff;
     rdata[3] = (msg->iana >> 16) & 0xff;
@@ -471,7 +481,7 @@ ipmi_emu_handle_msg(emu_data_t    *emu,
 }
 
 void
-ipmi_resend_atn(channel_t *chan)
+is_resend_atn(channel_t *chan)
 {
     lmc_data_t *mc = chan->mc;
 
@@ -480,7 +490,7 @@ ipmi_resend_atn(channel_t *chan)
 }
 
 msg_t *
-ipmi_mc_get_next_recv_q(channel_t *chan)
+is_mc_get_next_recv_q(channel_t *chan)
 {
     msg_t *rv;
 
@@ -712,25 +722,25 @@ chan_stop_cmd(channel_t *chan, int do_it_now)
 }
 
 channel_t **
-ipmi_mc_get_channelset(lmc_data_t *mc)
+is_mc_get_channelset(lmc_data_t *mc)
 {
     return mc->channels;
 }
 
 ipmi_sol_t *
-ipmi_mc_get_sol(lmc_data_t *mc)
+is_mc_get_sol(lmc_data_t *mc)
 {
     return &mc->sol;
 }
 
 unsigned char
-ipmi_mc_get_ipmb(lmc_data_t *mc)
+is_mc_get_ipmb(lmc_data_t *mc)
 {
     return mc->ipmb;
 }
 
 int
-ipmi_mc_users_changed(lmc_data_t *mc)
+is_mc_users_changed(lmc_data_t *mc)
 {
     int rv = mc->users_changed;
     mc->users_changed = 0;
@@ -738,26 +748,26 @@ ipmi_mc_users_changed(lmc_data_t *mc)
 }
 
 user_t *
-ipmi_mc_get_users(lmc_data_t *mc)
+is_mc_get_users(lmc_data_t *mc)
 {
     return mc->users;
 }
 
 pef_data_t *
-ipmi_mc_get_pef(lmc_data_t *mc)
+is_mc_get_pef(lmc_data_t *mc)
 {
     return &mc->pef;
 }
 
 startcmd_t *
-ipmi_mc_get_startcmdinfo(lmc_data_t *mc)
+is_mc_get_startcmdinfo(lmc_data_t *mc)
 {
     return &mc->startcmd;
 }
 
 int
-ipmi_mc_alloc_unconfigured(sys_data_t *sys, unsigned char ipmb,
-			   lmc_data_t **rmc)
+is_mc_alloc_unconfigured(sys_data_t *sys, unsigned char ipmb,
+			 lmc_data_t **rmc)
 {
     lmc_data_t *mc;
     unsigned int i;
@@ -889,7 +899,7 @@ ipmi_emu_add_mc(emu_data_t    *emu,
     int            i;
     sys_data_t     *sys = emu->sysinfo;
 
-    i = ipmi_mc_alloc_unconfigured(sys, ipmb, &mc);
+    i = is_mc_alloc_unconfigured(sys, ipmb, &mc);
     if (i)
 	return i;
 
@@ -975,7 +985,7 @@ ipmi_emu_add_mc(emu_data_t    *emu,
 	ipmi_register_child_quit_handler(&mc->child_quit_handler);
 	mc->tick_handler.info = mc;
 	mc->tick_handler.handler = handle_tick;
-	ipmi_register_tick_handler(&mc->tick_handler);
+	emu->sysinfo->register_tick_handler(&mc->tick_handler);
 	mc->channels[15]->start_cmd = chan_start_cmd;
 	mc->channels[15]->stop_cmd = chan_stop_cmd;
     }
@@ -986,97 +996,7 @@ ipmi_emu_add_mc(emu_data_t    *emu,
 }
 
 void
-ipmi_mc_set_device_id(lmc_data_t *mc, unsigned char device_id)
-{
-    mc->device_id = device_id;
-}
-
-unsigned char
-ipmi_mc_get_device_id(lmc_data_t *mc)
-{
-    return mc->device_id;
-}
-
-void
-ipmi_set_has_device_sdrs(lmc_data_t *mc, unsigned char has_device_sdrs)
-{
-    mc->has_device_sdrs = has_device_sdrs;
-}
-
-unsigned char
-ipmi_get_has_device_sdrs(lmc_data_t *mc)
-{
-    return mc->has_device_sdrs;
-}
-
-void
-ipmi_set_device_revision(lmc_data_t *mc, unsigned char device_revision)
-{
-    mc->device_revision = device_revision;
-}
-
-unsigned char
-ipmi_get_device_revision(lmc_data_t *mc)
-{
-    return mc->device_revision;
-}
-
-void
-ipmi_set_major_fw_rev(lmc_data_t *mc, unsigned char major_fw_rev)
-{
-    mc->major_fw_rev = major_fw_rev;
-}
-
-unsigned char
-ipmi_get_major_fw_rev(lmc_data_t *mc)
-{
-    return mc->major_fw_rev;
-}
-
-void
-ipmi_set_minor_fw_rev(lmc_data_t *mc, unsigned char minor_fw_rev)
-{
-    mc->minor_fw_rev = minor_fw_rev;
-}
-
-unsigned char
-ipmi_get_minor_fw_rev(lmc_data_t *mc)
-{
-    return mc->minor_fw_rev;
-}
-
-void
-ipmi_set_device_support(lmc_data_t *mc, unsigned char device_support)
-{
-    mc->device_support = device_support;
-}
-
-unsigned char
-ipmi_get_device_support(lmc_data_t *mc)
-{
-    return mc->device_support;
-}
-
-void
-ipmi_set_mfg_id(lmc_data_t *mc, unsigned char mfg_id[3])
-{
-    memcpy(mc->mfg_id, mfg_id, 3);
-}
-
-void
-ipmi_get_mfg_id(lmc_data_t *mc, unsigned char mfg_id[3])
-{
-    memcpy(mfg_id, mc->mfg_id, 3);
-}
-
-void
-ipmi_set_product_id(lmc_data_t *mc, unsigned char product_id[2])
-{
-    memcpy(mc->product_id, product_id, 2);
-}
-
-void
-ipmi_set_chassis_control_prog(lmc_data_t *mc, const char *prog)
+is_set_chassis_control_prog(lmc_data_t *mc, const char *prog)
 {
     mc->chassis_control_prog = prog;
 }
